<?php

namespace Drupal\mercury_editor\Controller;

use Drupal\node\NodeTypeInterface;
use Drupal\Core\Ajax\AjaxHelperTrait;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Entity\RevisionableInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\mercury_editor\MercuryEditorTempstore;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\Controller\EntityViewController;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class EditTrayController.
 *
 * Refreshes the node being edited in the tray.
 */
class MercuryEditorController extends EntityViewController {

  use AjaxHelperTrait;
  use StringTranslationTrait;

  /**
   * The Mercury Editor Edit Tray tempstore service.
   *
   * @var \Drupal\mercury_editor\MercuryEditorTempstore
   */
  protected $tempstore;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The entity repository.
   *
   * @var \Drupal\Core\Entity\EntityRepositoryInterface
   */
  protected $entityRepository;

  /**
   * Creates a NodeViewController object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entity_repository
   *   The entity repository.
   * @param \Drupal\mercury_editor\MercuryEditorTempstore $tempstore
   *   The tempstore.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, RendererInterface $renderer, AccountInterface $current_user, EntityRepositoryInterface $entity_repository, MercuryEditorTempstore $tempstore) {
    parent::__construct($entity_type_manager, $renderer);
    $this->currentUser = $current_user;
    $this->entityRepository = $entity_repository;
    $this->tempstore = $tempstore;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('renderer'),
      $container->get('current_user'),
      $container->get('entity.repository'),
      $container->get('mercury_editor.tempstore_repository')
    );
  }

  /**
   * Edit an entity with the mercury editor edit tray.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request.
   * @param \Drupal\Core\Entity\ContentEntityInterface $mercury_editor_entity
   *   The entity.
   *
   * @return array
   *   The edit form.
   */
  public function editor(Request $request, ContentEntityInterface $mercury_editor_entity) {
    $type = $mercury_editor_entity->getEntityTypeId();
    $form_object = \Drupal::entityTypeManager()->getFormObject($type, 'mercury_editor');
    // Load the entity from storage when first loading the form.
    if ($request->getMethod() === 'GET' &&  $mercury_editor_entity->id()) {
      if ($mercury_editor_entity instanceof RevisionableInterface) {
        /** @var \Drupal\Core\Entity\RevisionableStorageInterface $storage */
        $storage = \Drupal::entityTypeManager()
          ->getStorage($mercury_editor_entity->getEntityTypeId());
        $revision_id = $storage
          ->getLatestRevisionId($mercury_editor_entity->id());
        $mercury_editor_entity = $storage
          ->loadRevision($revision_id);
      }
      else {
        $mercury_editor_entity = $this->entityTypeManager->getStorage($type)->load($mercury_editor_entity->id());
      }
    }
    if (empty($form_object)) {
      // @todo Throw an error.
    }
    $edit_form = $form_object->setEntity($mercury_editor_entity);
    return \Drupal::formBuilder()->getForm($edit_form);
  }

  /**
   * Preview an entity being edited with the mercury editor edit tray.
   */
  public function preview(ContentEntityInterface $mercury_editor_entity) {
    return $this->view($mercury_editor_entity);
  }

  /**
   * Access callback for using mercury editor.
   */
  public function access(ContentEntityInterface $mercury_editor_entity) {
    return $mercury_editor_entity->id()
      ? $mercury_editor_entity->access('update', NULL, TRUE)
      : $mercury_editor_entity->access('create', NULL, TRUE);
  }

}
